package hdlists_server_cluster;

# Author : Daniel Viard <dviard@mandrakesoft.com>
# Version 0.1

use strict;
use Term::ANSIColor;
use MDK::Common;
use cluster_commonconf;
use cluster_fonction_common;

# test root capa
AmIRoot();

sub print_info {
  print "\n";
  print " - Generate hdlists with : \n";
  if (-e cluster_commonconf::install_data()->{LOCK}) {
    print " setup_hdlists_server already running !\n";
    print ' check ' . cluster_commonconf::install_data()->{LOG_FILE} . "\n";
    print ' or remove ' . cluster_commonconf::install_data()->{LOCK} . "\n";
  exit 1;
  }
}

sub lock_hdlists {
  touch(cluster_commonconf::install_data()->{LOCK});
  sys('more ' . cluster_commonconf::install_data()->{BASE_DIR} . '/hdlists');
  sleep 1;
}

sub erase_old_hdlists {
  print " - Erasing old hdlists\n";
#  eval {rm_rf(cluster_commonconf::install_data()->{BASE_DIR} . '/hdlist*.cz')};
  eval { sys('rm -rf ' . cluster_commonconf::install_data()->{BASE_DIR} . '/hdlist*.cz') };
  eval { rm_rf(map { cluster_commonconf::install_data()->{BASE_DIR} . "/$_" } qw 'depslist.ordered provides compss') };
#  eval { rm_rf(cluster_commonconf::install_data()->{BASE_DIR} . '/synthesis.hdlist*.cz') };
  eval { sys('rm -rf ' . cluster_commonconf::install_data()->{BASE_DIR} . '/synthesis.hdlist*.cz') };
}

sub generate_hdlists {
  my $log = cluster_commonconf::install_data()->{LOG_FILE};

  print "\n";
  print " - Generating hdlists\n";
  print "   |- Log are in : $log\n";
  print "\n";
  sys("cp -f /dev/null $log");

  if (cluster_commonconf::install_data()->{XTERM} eq 'TRUE') {
    sys("xterm -title Rapport_s_gendislog -e tail -f $log &");
  }

  eval { sys('gendistrib --distrib ' . cluster_commonconf::install_data()->{INSTALLDIR} . "/ 2> $log") };
}

sub generate_version_file {
  my $INSTALLDIR = cluster_commonconf::install_data()->{INSTALLDIR};

  print "\n";
  print " - Generating VERSION file\n";
  my $DATE = cluster_commonconf::mysystem()->{DATE};
  output_p("$INSTALLDIR/VERSION", "Cluster Devel $DATE");
  cp_af("$INSTALLDIR/VERSION", "$INSTALLDIR/Mandrake/mdkinst/usr/share/VERSION");
}

sub print_cluster_distrib_info {
  print " - Size of Cluster distrib: \n";
  sys('du -hs ' . cluster_commonconf::install_data()->{INSTALLDIR});
  print " - Number of packages per directory\n";
  my $DIR = cluster_commonconf::install_data()->{INSTALLDIR} . '/Mandrake';
  foreach my $i (all($DIR)) {
    if (!member($i, qw(share mdkinst base))) {
      print "   |- In $i there are :" . listlength(all("$DIR/$i")) . "\n";
    }
  }
}

sub ajust_permission {
  print "\n";
  print ' - Adjusting permission on ' . cluster_commonconf::install_data()->{INSTALLDIR} . "\n";
  sys('chown ' . cluster_commonconf::install_data()->{USER_INSTALL_LOGIN} . '.' . cluster_commonconf::install_data()->{USER_INSTALL_GROUP} . ' ' . cluster_commonconf::install_data()->{INSTALLDIR} . ' -R');
}

sub unlock_hdlists {
  rm_rf(cluster_commonconf::install_data()->{LOCK});

  print " - You can upgrade hdlists on client with:
   |- gexec -n 0 urpmi.update -a
      or
   |- rshpn urpmi.update -a
      or simply use use:
   |- urpmi --parallel cluster rpmname_to_install
";
}

sub help_hdlists {
    print "
 HELP:
 |----------------------------------------|
 | generate   generate the hdlists        |
 | info       print info of configuration |
 |----------------------------------------|

";
}

sub gen_hdlists {

  print_info();
  lock_hdlists();
  erase_old_hdlists();
  generate_hdlists();
  generate_version_file();
  print_cluster_distrib_info();
  ajust_permission();
  unlock_hdlists();
}

sub main {
    my %opts = (
		'' => \&help_hdlists,
		generate => \&gen_hdlists,
		info => \&print_info,
		);

    if (my $f = $opts{$ARGV[0]}) {
	$f->();
    } else {
	print " ** Dont know what todo ** \n";
    }
}

1;
